=begin
# Author:         tomot
# Name :          2DSectionTool 
# Description :   Create a 2D Vertical or Horizontal Sections, using Standard North American Lumber dimensions. and Architectural Material Symbols.  
# Usage :         1. Choose: View (Top, Front, Right, Back, Left) this tells 2DSectionTool's in what view its STUFF is going to be drawn.
#                 2. Click: 2 Pick Points. This tells 2DSectionTool's at location the STUFF is going to be drawn. 
#                 3. 2DSectionTool: Can draws its STUFF over top of existing 3D Section Elements.
#                 2DSectionTool: Can't read your intentions. Be sure to change the settings in the dialog box before using 2DSectionTool.
#                 "The Default": 2DSectionTool's draws: a 2D Vertical Section using, 2x6 Studs, Dbl. plates, Batt Insulation, with default 1/2" Interior and Exterior finishes, in Front View.
#                 If you choose 2 vertical pick points (one above the other), "The Default" will be drawn, vertically.
#                 If you choose 2 horizontal Pick Points (one left, the other to the right), "The Default" will be drawn, horizontally. 
#                 Before using 2DSectionTool, always decide if your Section is going to be a Vertical Section using Front, Right, Back, or Left Views. Or Horizontal Section, which uses the Top View.
#                 Make the approriate setting changes first, othewise you may end up deleting unintended geometry. 
#                 2DSectionTool: will also draw walls or floors at any angle. It depends if your 2nd Click, if your 2nd Click is not vertical or horizontal, your section will be drawn at the angle between Click 1 and Click 2. 
#                 Example: You may want to Insulate only: Turn Draw Insulation ON, Turn all other Draw functions OFF. Now use 2DSectionTool to Insulate a Rafter Section assembly
#                 2DSectionTool will always automatically create a layer named "Layer0section" on which it draws the section stuff. 
# Date :          2011,07,02
# Type :          Tool
# Revised:        2011,07,21 changed name, added 2D floor stuff, Changed globals to Alpha numerics, hence retain the values the user last used. 
#                 2011,07,21 added views, for easier drawing of 2D stuff over 3D section elements  
#                 2011,07,31 added layer, draws all 2d stuff on its own layer
#-----------------------------------------------------------------------------------------------
=end
require 'sketchup.rb'
require 'examples/linetool.rb'

class TwoDSectionTool < LineTool

def initialize
  Sketchup::set_status_text "Select Top of Floor", SB_PROMPT
  $ip1 = nil
  $ip2 = nil
  $xdown = 0
  $zdown = 0
  
  # sets the default Floor settings
  $axCB39 = 0.5.inch if not $axCB39
  $xyOP94 = 16.inch if not $xyOP94
  $wtVM17 = 0.5.inch if not $wtVM17
  $mfTR28 = 5.5.inch if not $mfTR28
  $ptAS54 = "OFF" if not $ptAS54
  $kvLV28 = "ON" if not $kvLV28
  $pRKD94 = "ON" if not $pRKD94
  $oxMW12 = "Front" if not $oxMW12
  $otTY84 = "Dbl. Plate" if not $otTY84
  
  # Dialog box
  view = ["Top", "Front", "Right", "Back", "Left"] 
  insul = ["ON","OFF"]
  joist = ["ON","OFF"]
  floor = ["ON","OFF"]
  options = ["Header", "Dbl. Plate", "Both", "None"]
  enums = [view.join("|"), "", "", "", "", floor.join("|"),joist.join("|"), insul.join("|"), options.join("|")]
  prompts = ["Choose View", "Ext-Wall or Sub-floor Thick. ", "Studs or Joists or Insul Thick  ","Int-Wall or Ceiling Thick", "Center Spacing", "Draw: Wall or Floor", "Draw: Stud or Joist @ Centers", "Draw: Insulation", "Draw: Wall Options"]
  values = [$oxMW12, $axCB39, $mfTR28, $wtVM17, $xyOP94, $pRKD94, $ptAS54, $kvLV28, $otTY84]
  results = inputbox prompts, values, enums, "Wall-or-Floor Section parameters"
  
  return nil if not results
     $oxMW12, $axCB39, $mfTR28, $wtVM17, $xyOP94, $pRKD94, $ptAS54, $kvLV28, $otTY84 = results
  
  if( $oxMW12 == "Top" ) 
    Sketchup.send_action("viewTop:")
    $orient = Geom::Vector3d.new(0,0,1)
    $normal = Geom::Vector3d.new(0,0,1)
  end #if      
  if( $oxMW12 == "Front" ) 
    Sketchup.send_action("viewFront:")
    $orient = Geom::Vector3d.new(0,1,0)
    $normal = Geom::Vector3d.new(0,1,0)
  end #if  
   if( $oxMW12 == "Right" ) 
    Sketchup.send_action("viewRight:")
    $orient = Geom::Vector3d.new(1,0,0)
    $normal = Geom::Vector3d.new(1,0,0)
  end #if  
  if( $oxMW12 == "Back" ) 
    Sketchup.send_action("viewBack:")
    $orient = Geom::Vector3d.new(0,1,0)
    $normal = Geom::Vector3d.new(0,1,0)
  end #if         
  if( $oxMW12 == "Left" ) 
    Sketchup.send_action("viewLeft:")
    $orient = Geom::Vector3d.new(1,0,0)
    $normal = Geom::Vector3d.new(1,0,0)  
  end #if  

end #end initialize

# Create new geometry when the user has selected two points.
def create_geometry(p1, p2, view)
 
   # create a layer on which to draw the 2DSection stufff
    model = Sketchup.active_model
    layers = model.layers
    layer_name = model.active_layer.name + "section"
    section_layer = layers[layer_name]
    if (not section_layer)
        section_layer = layers.add(layer_name)
        section_layer.visible = true;
    end
    model.active_layer = section_layer
       
    model.start_operation "Create Wall<>Floor Section"
    entities = model.active_entities
            
    # creates a new point from p1 in the direction of p2-p3 with length d
    # params are Point3d, 2 vertices, a length, returns a Point3d
    def translate(p1, p2, p3, d)
        v = p3 - p2
        v.length = d
        trans=Geom::Transformation.translation(v)
        return p1.transform(trans)
    end 
 
    if( $axis == "Yes" )
      view.model.entities.add_cline(p1,p2)
    end  
    
    pt1 = @ip1.position 
    pt2 = @ip2.position 
    pt2 = p2
        
    $pt1=Geom::Point3d.new(pt1[0],pt1[1],pt1[2])
    $pt2=Geom::Point3d.new(pt2[0],pt2[1],pt2[2])
    
    $vecz = Geom::Vector3d.new(0, 0, 1) 
    $vecx = Geom::Vector3d.new(1, 0, 0) 
    $vecy = Geom::Vector3d.new(0, 1, 0) 
    
    #...distance btwn. input points $pt1 & $pt2 (width of Floorsection)
    $vec=$pt2.distance $pt1
     
    $pt3=translate($pt1, $pt1, $pt2, 1.5) 
    $pt4=translate($pt1, $pt1, $pt2, $vec-1.5) 
    
    #...calculate new point pt11, $mfTR28 + $axCB39 + $wtVM17 and at the same angle plus 90 degrees from pt1
    vec = @ip2.position - $pt1
    ang = (180.degrees / 2.0)
    vec.length = $mfTR28 + $axCB39 + $wtVM17
    translate = Geom::Transformation.translation(vec)
    rotate = Geom::Transformation.rotation( $pt1, $orient, ang)
    trans = $pt1.transform(translate)
    $pt11 = trans.transform(rotate)
       
    #...caclulate new point pt22, $mfTR28 + $axCB39 + $wtVM17 and at the same angle plus 90 degrees from pt2
    vec = @ip1.position - $pt2
    ang = (180.degrees / 2.0)
    vec.length = $mfTR28 + $axCB39 + $wtVM17
    translate = Geom::Transformation.translation(vec)
    rotate = Geom::Transformation.rotation( $pt2, $orient, (0-ang))
    trans = $pt2.transform(translate)
    $pt22 = trans.transform(rotate)
    
    #...without this evaluation arcs will not rotate correctly within the rectangle
    $vecx = $pt22 - $pt11
    $vecy = $pt11 - $pt1
            
    #base=entities.add_face($pt1, $pt2, $pt22, $pt11) # basic rectangle 
    
    #...calculate points for subfloor   
    $pt33=translate($pt1, $pt1, $pt11, $axCB39)
    $pt44=translate($pt2, $pt2, $pt22, $axCB39)
    #...calculate points for joist depth
    $pt3=translate($pt1, $pt1, $pt11, $mfTR28+$axCB39)
    $pt4=translate($pt2, $pt2, $pt22, $mfTR28+$axCB39)
    #...calculate points for joist thickness
    $pt33a=translate($pt33, $pt33, $pt44, 1.5)
    $pt3a=translate($pt3, $pt3, $pt4, 1.5)
    #...calculate points for last joist thickness
    $pt44a=translate($pt44, $pt44, $pt33, 1.5)
    $pt44b=translate($pt4, $pt4, $pt3, 1.5)
    
   
    #...calculate insulation arcs
    $pt5=translate($pt3, $pt3, $pt4, 2.5) 
    $pt55=translate($pt33, $pt33, $pt44, 2.5) 
    $pt6=translate($pt3, $pt3, $pt4, 3.5) 
    $pt66=translate($pt33, $pt33, $pt44, 3.5) 
    $pt7=translate($pt3, $pt3, $pt4, $vec-3.5) 
    $pt77=translate($pt33, $pt33, $pt44, $vec-3.5) 
    $pt5500=translate($pt55, $pt55, $pt5, +1)
    $pt550=translate($pt55, $pt55, $pt5, $mfTR28-1)
    $pt330=translate($pt33a, $pt33a, $pt3a, $mfTR28-1)
    $pt660=translate($pt66, $pt66, $pt6, $mfTR28-1)
    $pt3300=translate($pt33a, $pt33a, $pt3a, +1)
    $pt6600=translate($pt66, $pt66, $pt6, +1)
    #...computer points for header
    $pt8=translate($pt3, $pt3, $pt4, $mfTR28+1.5)
    $pt88=translate($pt33, $pt33, $pt44, $mfTR28+1.5)
    $pt3c=translate($pt3a, $pt3a, $pt33a, 1.5)
    $pt3d=translate($pt3a, $pt3a, $pt33a, 3.0)
    $pt8c=translate($pt8, $pt8, $pt88, 1.5)
    $pt8d=translate($pt8, $pt8, $pt88, 3.0)
    
if( $pRKD94 != "OFF" )    
    #...define a new group
    group=entities.add_group
    entities=group.entities
       
    #...draw subfloor line
    base=entities.add_line($pt1, $pt2)
    #...draw top joist line
    base=entities.add_line($pt33, $pt44)
    #...draw btm joist line
    base=entities.add_line($pt3, $pt4) 
    #...draw ceiling line
    base=entities.add_line($pt11, $pt22) 
    
    group=entities.add_group
    entities=group.entities
    #...draw 1st joist
    base=entities.add_face($pt33, $pt33a, $pt3a, $pt3) 
    base=entities.add_line($pt33, $pt3a)
    base=entities.add_line($pt33a, $pt3)
    
    group=entities.add_group
    entities=group.entities
    #...draw last joist
    base=entities.add_face($pt44, $pt4, $pt44b, $pt44a) 
    base=entities.add_line($pt44, $pt44b)
    base=entities.add_line($pt44a, $pt4)
end  # if     

if( $otTY84 == "Dbl. Plate" )   
    #...draw top plate
    base=entities.add_face($pt77, $pt44a, $pt44b, $pt7) 
    base=entities.add_line($pt77, $pt44b)
    base=entities.add_line($pt7, $pt44a)
  end # if
  
  if( $otTY84 == "Header" ) 
    group = entities.add_group
    entities = group.entities
    #...create header
    base=entities.add_face($pt3a, $pt8, $pt8c, $pt3c)
    base=entities.add_face($pt3c, $pt8c, $pt8d, $pt3d)
    base=entities.add_line($pt3c, $pt8)
    base=entities.add_line($pt3d, $pt8c)
  end # if 
  
  if( $otTY84 == "Both" ) 
    group = entities.add_group
    entities = group.entities
    #...draw top plate
    base=entities.add_face($pt77, $pt44a, $pt44b, $pt7) 
    base=entities.add_line($pt77, $pt44b)
    base=entities.add_line($pt7, $pt44a)
    #...create header
    base=entities.add_face($pt3a, $pt8, $pt8c, $pt3c)
    base=entities.add_face($pt3c, $pt8c, $pt8d, $pt3d)
    base=entities.add_line($pt3c, $pt8)
    base=entities.add_line($pt3d, $pt8c)
  end # if 

if( $ptAS54 != "OFF" )  

    #...get the number of joists required for the related $vec/$xyOP94
    (($vec/$xyOP94)+1).to_i 
    
    definitions=model.definitions
    adding=definitions.add
    entities=adding.entities
    
    #...draw 1st joist
    # group=model.active_entities.add_group(base) # does nothing new
    base=entities.add_face($pt33, $pt33a, $pt3a, $pt3) 
    base=entities.add_line($pt33, $pt3a)
    base=entities.add_line($pt33a, $pt3)
    
       
    #...transform joist location 
    t = Geom::Transformation.translation(Geom::Vector3d.new(0, 0, 0))
    entities=model.active_entities
    
    entities.add_instance(adding, t)
    
    #...copy joists to their new locations
    i = 1
    while i < (($vec/$xyOP94)+1).to_i 
       # Transformation
       i = i + 1
       vec = $pt44b - $pt3a # width betweeen which joists are to be drawn
       vec.length = $xyOP94*(i-1)
       t=Geom::Transformation.translation(Geom::Vector3d.new(0,$xyOP94*(i-1), 0))
       t=Geom::Transformation.translation(vec)
       entities.add_instance(adding, t)
    end
end  # if 
          
if( $kvLV28 != "OFF" ) 

    #...distance btwn. joists for insulation
    $vec1=$pt3a.distance $pt44a #ok
    #...repeating insulation batt width
    $vec4=$pt6.distance $pt3a #ok
    
    #...get the number of insul batts required for the floor width $vec1/$vec4
    ($vec1/$vec4).to_i 
            
    definitions=model.definitions
    adding=definitions.add
    entities=adding.entities
    
    #...draw repeating 3 arcs for first Insul Batt
    #...1.draw the 180 degree insulation arc
    center = Geom::Point3d.new $pt550
    start_a = 0.0
    end_a = 180.degrees
    edgearray = entities.add_arc center, $vecx, $normal, 1, start_a, end_a
    edge = edgearray[0]
    arccurve = edge.curve
    end_angle = arccurve.end_angle
    #...2.draw the 90 degree insulation arc
    center = Geom::Point3d.new $pt3300 # new center location 90 degree arc  
    start_a = 0.0
    end_a = 180.degrees/2-(1.5707963267949)*2
    edgearray = entities.add_arc center, $vecx, $normal, 1, start_a, end_a  
    edge = edgearray[0]
    arccurve = edge.curve
    end_angle = arccurve.end_angle
    #...3.draw the 90 degree insulation arc
    center = Geom::Point3d.new $pt6600 # new center location 90 degree arc
    start_a = 0.0
    end_a = 180.degrees/2
    edgearray = entities.add_arc center, $vecx, $normal, -1, start_a, end_a
    edge = edgearray[0]
    arccurve = edge.curve
    end_angle = arccurve.end_angle
    
    base=entities.add_line($pt5500, $pt330)
    base=entities.add_line($pt5500, $pt660)
    
    #...transform Insul Batts location  
    t=Geom::Transformation.translation(Geom::Vector3d.new(0, 0, 0))
    entities=model.active_entities
    
    entities.add_instance(adding, t)
    
    #...copy Insul Batts to their new locations
    i = 1
    while i < ($vec1/$vec4).to_i 
       # Transformation
       i = i + 1
       vec = $pt44b - $pt3a
       vec.length = $vec4*(i-1)
       t=Geom::Transformation.translation(Geom::Vector3d.new(0,$vec4*(i-1), 0))
       t=Geom::Transformation.translation(vec) #without this transformation batts are not drawn along the wall
       entities.add_instance(adding, t)
    end  
end # if 

    #...reset layer back to default layer [0]
	layers = model.layers
    activelayer = model.active_layer=layers[0]
	layer=model.active_layer
   
    #self.reset # next click will call 2DSectionTool again without going back to menu  
    model.commit_operation
end

#Draw the geometry
def draw_geometry(pt1, pt2, view)
    view.draw_line(pt1, pt2)
end

end # class TwoDSectionTool

#-----------------------------------------------------------------------------
def floorsectiontool
    Sketchup.active_model.select_tool TwoDSectionTool.new
end

if( not file_loaded?("2DSectionTool.rb") )
    add_separator_to_menu("Plugins")
    UI.menu("Plugins").add_item("2DSectionTool") { Sketchup.active_model.select_tool TwoDSectionTool.new }
end
#-----------------------------------------------------------------------------
file_loaded("2DSectionTool.rb")
